"""Inspection pass types."""


class PassType:
    """Codes for PIGPASS."""

    COMPLEX = '2088658'
    COMPLEX_SKS = '2088661'
    COMPLEX_NAV = '5345726'
    NAVIGATE = '990003167701'
    CALIPER = '2088660'
    TFI = '2088662'
    ULTRASONIC = '2088663'
    EXPERIMENTAL = '2088664'
    EMAP = '6943277'



"""Infotech IPL XML."""
from lxml import etree as ET
from . import InfotechBase


class Typeobj:
    """Attributes of the TYPEOBJS IPL Infotech XML section."""

    Section = 'TYPEOBJS'
    Item = 'TYPEOBJ'
    SubItem = 'TITLE'
    Id = 'IDTYPEOBJ'


class Kamera:
    """Options for IPL_INSPECT.KZ_TYPE and KP_TYPE attributes."""

    Permanent = 1  # стационарная камера
    Temporary = 2  # временная камера
    Border = 3  # граница ЭО
    Armature = 4  # трубопроводная арматура
    Other = 5  # прочее (расшифровать)


class Inspect:
    """Attributes of the IPL_INSPECT Infotech XML item."""

    Title = 'IPL_INSPECT'

    Pipeline = "NLCH"  # "МГ Челябинск-Петровск"
    Place = "PLACE"  # "923,572-924 км (участок №1 L=195м)"
    L1 = "L1"  # "923,572"
    KzType = "KZ_TYPE"  # "2"
    L2 = "L2"  # "924"
    KpType = "KP_TYPE"  # "2"
    Isp = "ISP"  # "Газпроект-ДКР, ООО (г.Санкт-Петербург)"
    Start = "INSPECTION_START_DATE"  # "01.06.2018"
    End = "INSPECTION_END_DATE"  # "01.06.2018"


class Defect:
    """DEFECTS section."""

    Section = 'DEFECTS'
    Title = 'DEF'

    TypeId = "IDTYPEOBJ"  # "990004698869"
    Dist = "ODOMETER"  # "145"
    LOtch = "L_OTCH"  # "170"
    WOtch = "W_OTCH"  # "309"
    Vmin = "V_MIN_OTCH"  # "5"
    Vmax = "V_MAX_OTCH"  # "5"
    Orient1 = "ORIENT1"  # "7.2"
    Orient2 = "ORIENT2"  # "8.1"
    NumDef = "NUMDEF"  # "2"
    Rem = "REM"  # ""
    Kbd = "KBD"  # "0.70"
    Pbez = "PBEZ"  # ""
    TimeLimit = "TIME_LIMIT"  # "23.686"
    PbezPercent = "PBEZ_PERCENT"  # ""
    Method = "METHOD"  # "6907370"


class LineObj:
    """LINEOBJS section."""

    Section = 'LINEOBJS'
    Title = 'PLOBJ'

    TypeId = "IDTYPEOBJ"  # "990006537229"
    Dist = "ODOMETER"  # "0"
    Marker = "NAME_MARKER"  # "М1"
    Lch = "L_LCH"  # ""
    Rem = "REM"  # "(лоток запуска) 924 км"


class Weld:
    """WELDS section."""

    Section = 'WELDS'
    Title = 'WLD'

    TypeId = "IDTYPEOBJ"  # "2097791"
    Dist = "ODOMETER"  # "0"
    Num = "NUM_TUBE"  # "1"
    DlTube = "DL_TUBE"  # "752"
    Thick = "THICK"  # "19.5"
    Psh1 = "PSH1"  # "11.6"
    Psh2 = "PSH2"  # ""
    Rem = "REM"  # ""


class PigPass:
    """PIGPASS section."""

    Section = 'PIGPASS'
    Title = 'PASS'

    TypeId = "IDTYPEOBJ"  # "0"

    Date1 = "DATE1"  # "25.05.2018 10:58:00"
    Date2 = "DATE2"  # "25.05.2018 11:10:00"
    Speed = "SPEED_AVERAGE"  # "0.26"
    Rem = "REM"  # ""
    Manufact = "MANUFACTURER"  # "Саратоворгдиагностика"
    ManufactYear = "MANUFACT_DATE"  # "2010"
    PigType = "PIGTYPE"  # "990005096296"
    ObslType = "OBSLTYPE"  # "2"


class Infotech(InfotechBase):
    """Infotech IPL XML file."""

    def __init__(self):
        """Make new instance."""
        attribs = {
          Inspect.Pipeline: "",
          Inspect.Place: "",
          Inspect.L1: "",
          Inspect.KzType: "",
          Inspect.L2: "",
          Inspect.KpType: "",
          Inspect.Isp: "",
          Inspect.Start: "",
          Inspect.End: "",
        }
        self.root = ET.Element(Inspect.Title, attribs)
        self.typobjs = ET.SubElement(self.root, Typeobj.Section)
        self.defekts = ET.SubElement(self.root, Defect.Section)
        self.lineobjs = ET.SubElement(self.root, LineObj.Section)
        self.welds = ET.SubElement(self.root, Weld.Section)
        self.pigpass = ET.SubElement(self.root, PigPass.Section)

    def set_ipl(self, attr_dict):
        """Set attributes for IPL section."""
        for key, val in attr_dict.items():
            self.root.attrib[key] = val

    def add_pigpass(self, attr_dict):
        """Add PASS item with given attributes to PIGPASS section."""
        attribs = {
          PigPass.TypeId: "",
          PigPass.Date1: "",
          PigPass.Date2: "",
          PigPass.Speed: "",
          PigPass.Rem: "",
          PigPass.Manufact: "",
          PigPass.ManufactYear: "",
          PigPass.PigType: "",
          PigPass.ObslType: "",
        }
        pigpass = ET.SubElement(self.pigpass, PigPass.Title, attribs)
        for key, val in attr_dict.items():
            pigpass.attrib[key] = val

        return pigpass

    def add_weld(self, attr_dict):
        """Add WLD item with given attributes to WELDS section."""
        attribs = {
          Weld.TypeId: "",
          Weld.Dist: "",
          Weld.Num: "",
          Weld.DlTube: "",
          Weld.Thick: "",
          Weld.Psh1: "",
          Weld.Psh2: "",
          Weld.Rem: "",
        }
        weld = ET.SubElement(self.welds, Weld.Title, attribs)
        for key, val in attr_dict.items():
            weld.attrib[key] = val

        return weld

    def add_lineobj(self, attr_dict):
        """Add PLOBJ item with given attributes to LINEOBJS section."""
        attribs = {
          LineObj.TypeId: "",
          LineObj.Dist: "",
          LineObj.Marker: "",
          LineObj.Lch: "",
          LineObj.Rem: "",
        }
        lobj = ET.SubElement(self.lineobjs, LineObj.Title, attribs)
        for key, val in attr_dict.items():
            lobj.attrib[key] = val

        return lobj

    def add_defekt(self, attr_dict):
        """Add DEF item with given attributes to DEFECTS section."""
        attribs = {
          Defect.TypeId: "",
          Defect.Dist: "",
          Defect.LOtch: "",
          Defect.WOtch: "",
          Defect.Vmin: "",
          Defect.Vmax: "",
          Defect.Orient1: "",
          Defect.Orient2: "",
          Defect.NumDef: "",
          Defect.Rem: "",
          Defect.Kbd: "",
          Defect.Pbez: "",
          Defect.TimeLimit: "",
          Defect.PbezPercent: "",
          Defect.Method: "",
        }
        defekt = ET.SubElement(self.defekts, Defect.Title, attribs)
        for key, val in attr_dict.items():
            defekt.attrib[key] = val

        return defekt

    def add_types(self, it_dict):
        """Add given dict to TYPEOBJS."""
        for code, name in it_dict.items():
            item = ET.SubElement(self.typobjs, Typeobj.Item, {
              Typeobj.Id: code,
            })
            subitem = ET.SubElement(item, Typeobj.SubItem)
            subitem.text = name



"""Infotech TPO XML."""
from datetime import datetime
from lxml import etree as ET

from . import InfotechBase, InfotechError, it_date
from .codes.pypeline import PIPELINE_BY_NAMES


def get_abc(danger_code):
    """Return danger string for given code."""
    if danger_code in ["A", "a", "А", "а"]:
        return "Недопустимый (критический)"
    if danger_code in ["B", "b", "В", "в"]:
        return "Допустимый (ремонт)"
    # C
    return "Допустимый (малозначительный)"


class Typeobj:
    """Attributes of the TYPEOBJS TPO Infotech XML section."""

    Section = 'TYPEOBJS'
    Title = 'TYPEOBJ'

    Id = 'IDTYPEOBJ'
    Name = 'TITLE'


class Inspect:
    """Attributes of the TPO_INSPECT Infotech XML item."""

    Title = 'TPO_INSPECT'

    # ID трубопровода
    Idkc = "ID_KC"
    # Дата начала ВТД
    DateStart = "DATE_START"
    # Дата окончания ВТД
    DateEnd = "DATE_FINISH"
    ObslType = "OBSL_TYPE"
    Sposob = "SPOSOB_DIAGN"
    NumUch = "NUM_UCH"
    ObjDiagnUch = "OBJ_DIAGN_UCH"
    DistStartUch = "DIST_START_UCH"
    LengthUch = "LENGTH_UCH"
    IdInspect = "ID_INSPECT"
    MethodContr = "METHOD_CONTR"
    ErrLength = "ERR_LENGTH"
    ErrWidth = "ERR_WIDTH"
    ErrDepth = "ERR_DEPTH"


class Route:
    """Attributes of the ROUTE Infotech XML item."""

    Section = 'ROUTES'
    Title = 'ROUTE'

    Scheme = "ID_SCHEME"
    Idx = "IDX_ROUTE"
    Dfrom = "DFROM"
    Dto = "DTO"
    Plength_vto = "PLENGTH_VTO"
    Plength_vik = "PLENGTH_VIK"
    Plength_sv = "PLENGTH_SV"
    Plength_pv = "PLENGTH_PV"
    Flength_vto = "FLENGTH_VTO"
    Flength_vik = "FLENGTH_VIK"
    Flength_sv = "FLENGTH_SV"
    Flength_pv = "FLENGTH_PV"
    Descr = "DESCRIPTION"


class Elem:
    """Attributes of the ELEM Infotech XML item."""

    Title = 'ELEM'

    Num = "N_ELEM"  # "1.2"
    Vid = "VID_ELEM"  # "Сварное соединение"
    Oboznach = "OBOZNACH_ELEM"
    Name = "NAME_ELEM"
    Rasst1 = "RASST_ELEM1"
    Rasst2 = "RASST_ELEM2"
    Orient1 = "ORIENT1"
    Orient2 = "ORIENT2"
    Shift = "SHIFT"
    ThickMin = "THICK_ELEM_MIN"  # "17.3"
    ThickMax = "THICK_ELEM_MAX"  # "17.3"
    DiamMax = "DIAM_ELEM_MAX"  # "1400"
    Length = "LENGTH_ELEM"


class Defect:
    """Attributes of the DEFECT Infotech XML item."""

    Title = 'DEFECT'

    Num = "N_DEF"  # "2"
    Elem = "ELEMENT"  # "1.51"
    Kind = "ID_KIND"  # "5215283"
    Length1 = "LEN_DEF1"
    Length2 = "LEN_DEF2"
    Orient1 = "ORIENT1"  # "8"
    Orient2 = "ORIENT2"  # "9"
    Angle = "ANGLE_DEF"
    Length = "LENGTH"
    Width = "WIDTH"  # 248"
    Depth = "DEPTH"
    RelDepth = "REL_DEPTH"
    Rang = "RANG_OPASN"  # "Недопустимый (критический)"
    Recom = "RECOM"
    Meropr = "VID_MEROPR"  # "Замена"


class Infotech(InfotechBase):
    """Infotech TPO XML file."""

    def __init__(self):
        """Make new instance."""
        attribs = {
          Inspect.Idkc: "",
          Inspect.DateStart: "",
          Inspect.DateEnd: "",
          Inspect.ObslType: "",
          Inspect.Sposob: "",
          Inspect.NumUch: "",
          Inspect.ObjDiagnUch: "",
          Inspect.DistStartUch: "",
          Inspect.LengthUch: "",
          Inspect.IdInspect: "",
          Inspect.MethodContr: "",
          Inspect.ErrLength: "",
          Inspect.ErrWidth: "",
          Inspect.ErrDepth: "",
        }
        self.root = ET.Element(Inspect.Title, attribs)
        self.typobjs = ET.SubElement(self.root, Typeobj.Section)
        self.routes = ET.SubElement(self.root, Route.Section)

    def set_titul(
      self,
      pipeline_name,
      date_start,
      date_end,
      obsl_type,  # DefaultID.OBSL_TYPE
      sposob,  # DefaultID.SPOSOB_DIAGN
      id_inspect,  # DefaultID.ID_INSPECT
      method_contr,  # DefaultID.METHOD_CONTR
      type_dict
    ):
        """Set inspection attributes values."""
        pipeline_id = PIPELINE_BY_NAMES.get(pipeline_name)
        if pipeline_id is None:
            raise InfotechError("Газопровод '{}' не найден в словаре Инфотех.".format(pipeline_name))

        err = ""
        if not isinstance(date_start, datetime):
            err += "Неверная дата начала: '{}'\n".format(date_start)
        if not isinstance(date_end, datetime):
            err += "Неверная дата окончания: '{}'\n".format(date_end)

        if err:
            raise InfotechError(err)

        s_start = it_date(date_start)
        s_end = it_date(date_end)

        self.root.attrib[Inspect.Idkc] = pipeline_id
        self.root.attrib[Inspect.DateStart] = s_start
        self.root.attrib[Inspect.DateEnd] = s_end
        self.root.attrib[Inspect.ObslType] = obsl_type
        self.root.attrib[Inspect.Sposob] = sposob
        self.root.attrib[Inspect.IdInspect] = id_inspect
        self.root.attrib[Inspect.MethodContr] = method_contr

        self.add_types({pipeline_id: pipeline_name})
        self.add_types(type_dict)

    def add_route(self, iid, length, scheme_file_name):
        """Add new route with given ID and length."""
        s_len = "{:0.1f}".format(length)
        attribs = {
          Route.Scheme: scheme_file_name,
          Route.Idx: str(iid),
          Route.Dfrom: self.root.attrib[Inspect.DateStart],
          Route.Dto: self.root.attrib[Inspect.DateEnd],
          Route.Plength_vto: "",
          Route.Plength_vik: "",
          Route.Plength_sv: "",
          Route.Plength_pv: "",
          Route.Flength_vto: s_len,
          Route.Flength_vik: s_len,
          Route.Flength_sv: s_len,
          Route.Flength_pv: s_len,
          Route.Descr: "",
        }
        route = ET.SubElement(self.routes, Route.Title, attribs)

        return route

    def set_total_length(self, length):
        """Set inspection attribute of total length."""
        if not isinstance(length, float):
            raise InfotechError("Неверная длина участка: '{}'\n".format(length))

        self.root.attrib[Inspect.LengthUch] = "{:0.1f}".format(length)

    def set_oboznach(self, elem, text):
        """Set attribute ELEM.OBOZNACH_ELEM with givem text."""
        elem.attrib[Elem.Oboznach] = text

    def set_sdt(self, elem, dist, orient1, orient2, length, diam, thick):  # pylint: disable=too-many-arguments
        """Set SDT attributes for ELEM."""
        if isinstance(dist, (int, float)):
            elem.attrib[Elem.Rasst1] = "{:0.1f}".format(round(float(dist), 1))

        if isinstance(orient1, (int, float)):
            elem.attrib[Elem.Orient1] = str(round(float(orient1)))

        if isinstance(orient2, (int, float)):
            elem.attrib[Elem.Orient2] = str(round(float(orient2)))

        if isinstance(thick, (int, float)):
            text = "{:0.1f}".format(round(float(thick), 1))
            elem.attrib[Elem.ThickMin] = text
            elem.attrib[Elem.ThickMax] = text

        if isinstance(diam, int):
            elem.attrib[Elem.DiamMax] = str(diam)

        if isinstance(length, int):
            elem.attrib[Elem.Length] = str(length)

    def add_element(self, route, num, name):
        """Add ELEM item."""
        attribs = {
          Elem.Num: num,
          Elem.Vid: name,
          Elem.Oboznach: "",
          Elem.Name: "",
          Elem.Rasst1: "",
          Elem.Rasst2: "",
          Elem.Orient1: "",
          Elem.Orient2: "",
          Elem.Shift: "",
          Elem.ThickMin: "",
          Elem.ThickMax: "",
          Elem.DiamMax: "",
          Elem.Length: "",
        }
        return ET.SubElement(route, Elem.Title, attribs)

    def add_defect(
      self, route, elem_num, num, iid, from_weld, orient1, orient2, length, width, depth, danger, action
    ):  # pylint: disable=too-many-arguments
        """Set inspection attribute of total length."""
        attribs = {
          Defect.Num: str(num),
          Defect.Elem: str(elem_num),
          Defect.Kind: iid,
          Defect.Length1: "",
          Defect.Length2: "",
          Defect.Orient1: "",
          Defect.Orient2: "",
          Defect.Angle: "",
          Defect.Length: "",
          Defect.Width: "",
          Defect.Depth: "",
          Defect.RelDepth: "",
          Defect.Rang: "",
          Defect.Recom: "",
          Defect.Meropr: "",
        }
        item = ET.SubElement(route, Defect.Title, attribs)

        if isinstance(from_weld, int):
            item.attrib[Defect.Length1] = str(from_weld)

        if isinstance(orient1, (int, float)):
            item.attrib[Defect.Orient1] = str(round(float(orient1)))

        if isinstance(orient2, (int, float)):
            item.attrib[Defect.Orient2] = str(round(float(orient2)))

        if isinstance(length, int):
            item.attrib[Defect.Length] = str(length)

        if isinstance(width, int):
            item.attrib[Defect.Width] = str(width)

        if isinstance(depth, (int, float)):
            item.attrib[Defect.Depth] = "{:0.1f}".format(round(float(depth), 1))

        if isinstance(danger, str):
            item.attrib[Defect.Rang] = get_abc(danger)

        if isinstance(action, str):
            if action not in ["-"]:
                item.attrib[Defect.Meropr] = action

        return item

    def add_types(self, it_dict):
        """Add given dict to TYPEOBJS."""
        for code, name in it_dict.items():
            ET.SubElement(self.typobjs, Typeobj.Title, {
              Typeobj.Id: code,
              Typeobj.Name: name,
            })



"""Base Infotech XML stuff."""
from lxml import etree as ET


def it_date(dtime):
    """Return string date in Infotech format for given datetime value."""
    return "{:02d}.{:02d}.{}".format(dtime.day, dtime.month, dtime.year)


class InfotechError(Exception):
    """Infotech exceptions."""


class InfotechBase:
    """Abstract base Infotech XML class."""

    root = None
    typobjs = None

    def save(self, file_name, encoding="windows-1251"):
        """Save XML string to file.

        Python >= 3.8
        https://stackoverflow.com/questions/62163221/creating-xml-file-with-a-specific-order-or-attributes-using-python
        """
        with open(file_name, 'wb') as out:
            out.write(ET.tostring(self.root, xml_declaration=True, encoding=encoding, pretty_print=True))
